<?php

namespace Drupal\csv_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;

/**
 * Plugin implementation of the 'file_generic' widget.
 *
 * @FieldWidget(
 *   id = "csv_file_generic",
 *   label = @Translation("Csv File"),
 *   field_types = {
 *     "csv_file"
 *   }
 * )
 */
class CsvFileWidget extends FileWidget implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['updateSettingsAccess'];
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    // Note: this changes the #process and #value_callback class to this class.
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    return $element;
  }

  /**
   * Form API callback: Processes a csv_file_generic field element.
   *
   * Expands the csv_file_generic type to include the settings field.
   *
   * This method is assigned as a #process callback in formElement() method.
   */
  public static function process($element, FormStateInterface $form_state, $form) {
    // Make sure FileWidget processing runs.
    $element = parent::process($element, $form_state, $form);
    $item = $element['#value'];

    $parents = $element['#parents'];
    $selector = $root = array_shift($parents);
    if ($parents) {
      $selector = $root . '[' . implode('][', $parents) . ']';
    }

    $element['settings'] = [
      '#type' => 'details',
      '#title' => t('Display Configuration'),
      '#open' => FALSE,
      '#weight' => 2,
    ];

    $element['settings']['paging'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable paging'),
      '#default_value' => !empty($item['settings']['paging']) ? 1 : 0,
      '#description' => t('Note: For larger CSVs, disabling paging can result in performance issues while the table is rendering.')
    ];

    $element['settings']['enableVerticalScroll'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable vertical scrolling in container'),
      '#default_value' => !empty($item['settings']['enableVerticalScroll']) ? 1 : 0,
      '#states' => [
        'visible' => [
          ':input[name^="' . $selector . '[settings][paging]"]' => ['checked' => FALSE],
        ],
      ],
    ];

    $element['settings']['scrollY'] = [
      '#type' => 'select',
      '#title' => t('Vertical Scroll Container Height'),
      '#options' => [
        '200px' => '200px',
        '300px' => '300px',
        '400px' => '400px',
        '500px' => '600px',
        '600px' => '700px',
        '800px' => '800px',
        '900px' => '900px',
        '1000px' => '1000px',
        '1100px' => '1100px',
        '1200px' => '1200px',
      ],
      '#default_value' => !empty($item['settings']['scrollY']) ? $item['settings']['scrollY'] : '400px',
      '#states' => [
        'visible' => [
          ':input[name^="' . $selector . '[settings][paging]"]' => ['checked' => FALSE],
          ':input[name^="' . $selector . '[settings][enableVerticalScroll]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $element['settings']['pageLength'] = [
      '#type' => 'select',
      '#title' => t('Initial Page Length'),
      '#options' => [
        5 => 5,
        10 => 10,
        25 => 25,
        50 => 50,
        75 => 75,
        100 => 100,
      ],
      '#default_value' => !empty($item['settings']['pageLength']) ? $item['settings']['pageLength'] : 10,
      '#states' => [
        'visible' => [
          ':input[name^="' . $selector . '[settings][paging]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $element['settings']['lengthChange'] = [
      '#type' => 'checkbox',
      '#title' => t('Allow end user to change number of rows to display'),
      '#default_value' => !empty($item['settings']['lengthChange']) ? 1 : 0,
      '#states' => [
        'visible' => [
          ':input[name^="' . $selector . '[settings][paging]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $element['settings']['searching'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable searching'),
      '#default_value' => !empty($item['settings']['searching']) ? 1 : 0,
    ];

    $element['settings']['responsive'] = [
      '#type' => 'radios',
      '#title' => t('Responsive Option'),
      '#options' => [
        'horizontal_scroll' => t("Enable horizontal scrolling if all columns don't fit on the page"),
        'childRow' => t("Show fields on the left side of the table that will fit on the screen horizontally and have an <strong>expansion button</strong> for visitors to see other fields that don't fit stacked vertically below the row"),
        'childRowImmediate' => t("Show fields on the left side of the table that will fit on the screen horizontally and <strong>automatically show</strong> other fields that don't fit stacked vertically below the row"),
      ],
      '#default_value' => !empty($item['settings']['responsive']) ? $item['settings']['responsive'] : 'childRow',
    ];

    $element['settings']['download'] = [
      '#type' => 'checkbox',
      '#title' => t('Display a download link'),
      '#default_value' => !empty($item['settings']['download']) ? 1 : 0,
    ];

    // Do not display description field as `Download Link Text` field overrides
    // it.
    if (!empty($element['description'])) {
      $element['settings']['downloadText'] = [
        '#type' => 'textfield',
        '#title' => t('Download Link Text'),
        '#default_value' => !empty($item['settings']['downloadText']) ? $item['settings']['downloadText'] : '',
        '#states' => [
          'visible' => [
            ':input[name^="' . $selector . '[settings][download]"]' => ['checked' => TRUE],
          ],
        ],
        '#description' => t('Leave this field blank to use filename.'),
      ];

      $element['description']['#access'] = FALSE;
    }

    $element['settings']['autolink'] = [
      '#type' => 'checkbox',
      '#title' => t('Automatically Convert Urls to Links.'),
      '#default_value' => !empty($item['settings']['autolink']) ? 1 : 0,
    ];

    return $element;

  }

  /**
   * Form API callback. Retrieves the value for the csv_file field element.
   *
   * This method is assigned as a #value_callback in formElement() method.
   */
  public static function value($element, $input, FormStateInterface $form_state) {
    $return = parent::value($element, $input, $form_state);

    // If this is a newly uploaded file, each setting will each be NULL.
    // We only need to check the responsive option, which should be a string.
    if (empty($return['settings']) || !$return['settings']['responsive']) {
      $return['settings'] = [
        'paging' => 1,
        'searching' => 1,
        'pageLength' => 5,
        'lengthChange' => 1,
        'responsive' => 'childRow',
        'download' => 1,
        'enableVerticalScroll' => 0,
        'scrollY' => '400px',
        'autolink' => 0,
      ];
    }
    else {
      // Some Drupal form elements convert integers to strings.
      // Assure integer values are saves as integers, not strings.
      $int_settings = [
        'paging',
        'searching',
        'pageLength',
        'lengthChange',
        'download',
        'enableVerticalScroll',
        'autolink',
      ];
      foreach ($int_settings as $key) {
        if (array_key_exists($key, $return['settings'])) {
          $return['settings'][$key] = (int) $return['settings'][$key];
        }
      }
    }

    if (array_key_exists('downloadText', $return['settings'])) {
      $return['description'] = $return['settings']['downloadText'];
    }

    return $return;
  }

}
